package Koha::Contrib::Tamil;
# ABSTRACT: Various scripts to manage Koha instances
$Koha::Contrib::Tamil::VERSION = '0.054';
use warnings;
use strict;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Koha::Contrib::Tamil - Various scripts to manage Koha instances

=head1 VERSION

version 0.054

=head1 DESCRIPTION

This module contains L<Tamil|http://www.tamil.fr> L<Koha Open Source
ILS|http://www.koha-community.org> contribution scripts. Those scripts are used
to manage Koha instances.

=head1 SEE ALSO

=over 4

=item *

L<koha-index>

=item *

L<koha-index-daemon>

=back

=head1 AUTHOR

Frédéric Demians <f.demians@tamil.fr>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2018 by Fréderic Démians.

This is free software, licensed under:

  The GNU General Public License, Version 3, June 2007

=cut
